/*
 * File Name: label.c
 */


/**
 * Copyright (C) 2011 Mackx.
 * All rights reserved.
 */

//----------------------------------------------------------------------------
// Include Files
//----------------------------------------------------------------------------

#include <glib.h>
#include <gtk/gtk.h>
#include <stdlib.h>

#include "xournal.h"

#include "label.h"

//----------------------------------------------------------------------------
// Type Declarations
//----------------------------------------------------------------------------


//----------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------
enum
{
   NAME_COLUMN,
   PAGE_COLUMN,
   N_COLUMNS
};


//----------------------------------------------------------------------------
// Static Variables
//----------------------------------------------------------------------------



//============================================================================
// Local Function Definitions
//============================================================================

static void tree_selection_activated_cb (GtkTreeView        *treeview,
						                 GtkTreePath        *path,
						                 GtkTreeViewColumn  *col,
						                 gpointer            userdata);


static void label_dialog_populate_tree_model (GtkTreeStore *store);


//============================================================================
// Functions Implementation
//============================================================================

// add a new label
void label_add_new(gchar *name, int page)
{
}

void label_split_text(struct Item *item)
{
    if (item == NULL) return;
    if (item->text == NULL) return;
    
   // Check if text is a special text
    if (g_ascii_strncasecmp(item->text, "label:", strlen("label:")) == 0)
    {
        // TODO: code leads to memory leak when there are multiple 'label:'s in the original text
        // save the original text in extended_text
        item->extended_text = item->text;
        // text will only contain the part that should be shown
        item->text = g_strdup(&item->extended_text[6]);
    }
}

// Show the label selection dialog
char *label_selection_dialog(const gchar          *title,
						     GtkWindow            *parent)
{
	GtkWidget *widget;
    GtkWidget *dialog;
	GtkTreeStore *store;
	GtkTreeViewColumn *column;
	GtkCellRenderer *renderer;
	GtkTreeSelection *select;

    // object hierarchy:
	//       dialog
	//         |
    /* Create the widgets */
    dialog = gtk_dialog_new_with_buttons (title,
                                          parent,
                                          GTK_DIALOG_DESTROY_WITH_PARENT,
										  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          NULL);
  	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
  	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
  	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 2); /* 2 * 5 + 2 = 12 */
    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area), 5);

	/* Create a model.  We are using the store model for now, though we
	* could use any other GtkTreeModel */
	store = gtk_tree_store_new (N_COLUMNS,
                                G_TYPE_STRING,
                                G_TYPE_UINT);

	/* custom function to fill the model with data */
	label_dialog_populate_tree_model (store);

	/* Create a view */
	widget = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
	g_signal_connect (G_OBJECT (widget), "row-activated",
					  G_CALLBACK (tree_selection_activated_cb),
					  NULL);

	/* The view now holds a reference.  We can get rid of our own
	* reference */
	g_object_unref (G_OBJECT (store));

	/* First column.. Position in the Desktop */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Name", renderer,
                                                       "text", NAME_COLUMN,
                                                       NULL);

	/* Add the column to the view. */
	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Page", renderer,
                                                       "text", PAGE_COLUMN,
                                                       NULL);

	/* Add the column to the view. */
	gtk_tree_view_append_column (GTK_TREE_VIEW (widget), column);

  	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), widget, TRUE, TRUE, 0);

  	gtk_widget_show (widget);

    printf("Showing Dialog\n");
  	if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
    	gtk_widget_destroy(dialog);
    	return NULL;
  	}
  	gtk_widget_destroy(dialog);

	return NULL;
}

static void label_dialog_populate_tree_model (GtkTreeStore *store)
{
	GtkTreeIter   iter;
    struct Page *pg, *tmppg;
    struct Layer *layer;
    struct Item *item;
    GList *pagelist, *layerlist, *itemlist, *list;
    guint page_no = 1;

    printf("entry:label_dialog_populate_tree_model\n");
    
    for (pagelist = journal.pages; pagelist!=NULL; pagelist = pagelist->next, page_no++) 
    {
        pg = (struct Page *)pagelist->data;
        for (layerlist = pg->layers; layerlist!=NULL; layerlist = layerlist->next) 
        {
            layer = (struct Layer *)layerlist->data;
            for (itemlist = layer->items; itemlist!=NULL; itemlist = itemlist->next) 
            {
                item = (struct Item *)itemlist->data;
                if ((item->type == ITEM_TEXT) && (item->extended_text != NULL))
                {
                    // search in extended_text for labels
                    if (g_ascii_strncasecmp(item->extended_text, "label:", strlen("label:")) == 0)
                    {
                        printf("Adding:%s [%d]\n", item->text, page_no);
                        gtk_tree_store_append (store, &iter, NULL);  /* Acquire an iterator */
                        gtk_tree_store_set (store, &iter,
                                            NAME_COLUMN, item->text,
                                            PAGE_COLUMN, page_no,
                                            -1);
                    }
                }
            }
        }
    }
}

// row double clicked
static void
tree_selection_activated_cb (GtkTreeView        *treeview,
						     GtkTreePath        *path,
						     GtkTreeViewColumn  *col,
						     gpointer            userdata)
{
    GtkTreeModel *model;
    GtkTreeIter   iter;
 
    model = gtk_tree_view_get_model(treeview);
 
    if (gtk_tree_model_get_iter(model, &iter, path))
    {
        gchar *name;
        guint page;
 
		gtk_tree_model_get (model, &iter, NAME_COLUMN, &name, 
                                          PAGE_COLUMN, &page,
										  -1);
 
        printf("Double-clicked row contains name %s [%d]\n", name, page);
        
        do_switch_page(page-1, TRUE, FALSE);

        g_free(name);
    }
}


